/**
  * i18n-plugin v0.1.4
  *
  * @link https://vue-formily.netlify.app/plugins/i18n
  * @source https://github.com/vue-formily/i18n-plugin
  * (c) 2021 An Ha
  * @license MIT
  */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.I18nPlugin = factory());
}(this, (function () { 'use strict';

  const _toString = Object.prototype.toString;
  /**
   * Strict object type check. Only returns true
   * for plain JavaScript objects.
   */

  function isPlainObject(obj) {
    return _toString.call(obj) === '[object Object]';
  }
  function isString(value) {
    return typeof value === 'string';
  }

  const rkey = /([^[\]]+)/g;
  /**
   * Find the first found value in objects or arrays by its path
   */

  function get(path, ...args) {
    const length1 = args.length;
    const paths = isString(path) ? path.split('.') : path;
    const length2 = paths.length;

    for (let i = 0; i < length1; i++) {
      let value = args[i];
      let j = -1;

      while (value && ++j < length2) {
        const fullName = paths[j];
        const m = fullName.match(rkey);

        if (m) {
          const length3 = m.length;

          for (let k = 0; k < length3; k++) {
            const name = m[k];

            if ((Array.isArray(value) || isPlainObject(value)) && name in value) {
              value = value[name];

              if (j === length2 - 1 && k === length3 - 1) {
                return value;
              }
            }
          }
        }
      }
    }

    return undefined;
  }
  /**
   * Creates a new array with all sub-array elements concatenated into it recursively up to the specified depth.
   */

  function flatArray(arr, deep = Infinity) {
    return deep > 0 ? arr.reduce((acc, val) => acc.concat(Array.isArray(val) ? flatArray(val, deep - 1) : val), []) : arr;
  }

  const simplePlaceholderRegex = /\{([^{}]+)\}/g;

  function formatString(format, data) {
    return format.replace(simplePlaceholderRegex, (_, path) => {
      const value = get(path, ...flatArray([data]));
      return '' + value;
    });
  }

  var stringFormat = {
    name: 'string-format',
    format: formatString,

    install(Objeto, options = {}) {
      this.options = { ...this.options,
        ...options
      };
      Objeto.prototype.$stringFormat = this;
    },

    options: {}
  };

  let _activeLocale = 'en-US';
  const _locales = {};
  var index = {
      switchLocale(locale) {
          if (!_locales[locale]) {
              throw new Error(`${locale} does not exist.`);
          }
          _activeLocale = locale;
      },
      addLocale(locale) {
          if (isPlainObject(locale)) {
              _locales[locale.code] = locale;
          }
      },
      removeLocale(locale) {
          delete _locales[locale];
      },
      getLocale(locale) {
          return _locales[locale];
      },
      translate(key, data, options = {}) {
          let locale = options.locale;
          if (isPlainObject(data) && data.formType === 'field') {
              const { i18n = {} } = data.options || {};
              locale = i18n.locale;
          }
          const loc = _locales[locale || _activeLocale] || {};
          const format = get(key, loc.resource);
          return stringFormat.format(format !== undefined ? format : key, flatArray([data, loc.localize]));
      },
      install(config, options = {}) {
          this.options = options;
          const { locales = [], locale = _activeLocale } = this.options;
          locales.forEach((locale) => this.addLocale(locale));
          if (locales.length && locales[locale]) {
              this.switchLocale(locale);
          }
          config.plugs.i18n = this;
      }
  };

  return index;

})));
