import { flatArray, isPlainObject, get } from '@vue-formily/util';
import stringFormat from '@vue-formily/string-format';
let _activeLocale = 'en-US';
const _locales = {};
export default {
  switchLocale(locale) {
    if (!_locales[locale]) {
      throw new Error(`${locale} does not exist.`);
    }

    _activeLocale = locale;
  },

  addLocale(locale) {
    if (isPlainObject(locale)) {
      _locales[locale.code] = locale;
    }
  },

  removeLocale(locale) {
    delete _locales[locale];
  },

  getLocale(locale) {
    return _locales[locale];
  },

  translate(key, data, options = {}) {
    let locale = options.locale;

    if (isPlainObject(data) && data.formType === 'field') {
      const {
        i18n = {}
      } = data.options || {};
      locale = i18n.locale;
    }

    const loc = _locales[locale || _activeLocale] || {};
    const format = get(key, loc.resource);
    return stringFormat.format(format !== undefined ? format : key, flatArray([data, loc.localize]));
  },

  install(config, options = {}) {
    this.options = options;
    const {
      locales = [],
      locale = _activeLocale
    } = this.options;
    locales.forEach(locale => this.addLocale(locale));

    if (locales.length && locales[locale]) {
      this.switchLocale(locale);
    }

    config.plugs.i18n = this;
  }

};